#ifndef JEU_HPP
#define JEU_HPP

#include <string>
#include "echiquier.hpp"
#include "enums.hpp"

/// @brief Classe représentant un Jeu
class Jeu{
	private:
		/// @brief Nom du jeu en cours
		std::string name;
		/// @brief Echiquier lié au jeu
		Echiquier echiquier;
		/// @brief Joueur qui doit jouer
		Color player{WHITE};
		/// @brief Message à afficher dans la console
		std::string message = "Que souhaitez vous faire ?"; 
	public:
		/// @brief Initialise la classe
		/// @param name Nom du jeu
		Jeu(std::string name);

		/// @brief Prépare le jeu en plaçant les pièces
		void setup();
		/// @brief Affiche l'échiquier ainsi que l'interface dans la sortie standard
		void print() const;
		/// @brief 
		/// @return 
		Color get_player() const { return player; }

		/// @brief Récupère la position canonique du jeu actuel
		/// @return Position canonique sous forme d'une chaîne de caractère
		std::string canonical() const;

		/// @brief Vérifie si le jeu est en cours, est en situation de pat ou de mat.
		/// @return Status du jeu
		Status check_status();

		/// @brief Met à jour le message dans la console avant la saisie de l'utilisateur
		/// @param message Message à afficher dans la console
		void set_message(std::string message){ this->message = message; }

		/// @brief Permet d'effectuer un coup
		/// @param from Case de destination
		/// @param to Case d'origine
		/// @return true si le coup a eu lieu, false sinon
		bool move(Square from, Square to);

		/// @brief Permet d'effectuer un roque
		/// @param big true si grand roque, false si petit
		/// @return true si le roque a eu lieu, false sinon
		bool castling(bool big);
};

#endif