#ifndef PIECES_PION_HPP
#define PIECES_PION_HPP

#include "piece.hpp"

/// Pion dans le jeu d'échec
class Pion : public Piece {
	private:
		/// @brief Fonction exécutant les opérations nécessaires au bon fonctionnement de la promotion
		void handle_promote();

		/// Permet de voir si le pion peut faire une prise en passant
		/// @param target_piece Pièce ciblée par la prise en passant
		bool can_en_passant(Piece* target_piece) const;
	public:
		/// @brief Vérifie si un mouvement de la case origine à la case destination est valide
		/// @param org Case d'origine
		/// @param dest Destination
		/// @return Vrai si le mouvement est légal, faux sinon
		bool est_mouvement_legal(Square org, Square dest) const override;

		/// @brief Vérifie si la pièce est une menace pour l'adversaire
		/// @param org Position d'origine
		/// @param dest Position de destination
		/// @return true si la pièce est une menace, false sinon
		bool est_menace(Square org, Square dest) const override;

		/// @brief Fonction appelée quand une pièce bouge
		/// @param from Case d'origine
		void move_event(Square from) override;
		using Piece::Piece;
};

#endif