#include "pieces/piece.hpp"

#include <utility>
#include "echiquier.hpp"

bool Piece::est_mouvement_legal(Square org, Square dest) const{
	int row_dest, col_dest;
	dest.get_row_col(row_dest, col_dest);

	// Cas où on dépasse l'échiquier
	if(row_dest < 0 || row_dest > 7 || col_dest < 0 || col_dest > 7) return false;
	// Cas où une pièce de la même couleur est présente
	Piece* piece = echiquier->get_piece(dest);
	if(piece != nullptr && piece->get_color() == this->color) return false;
	return true;
}

bool Piece::est_menace(Square org, Square dest) const {
	return false;
}

Piece::~Piece() = default;

bool Piece::operator==(const Piece &other) const {
	return id == other.id && color == other.color;
}

Piece::Piece(std::string name, 
             std::string canon_name,
             Color color,
             int id,
             Square start_pos,
             Echiquier* const echiquier) : name(std::move(name)), canon_name(std::move(canon_name)), color(color), id(id), pos(start_pos), echiquier(echiquier) {}

std::string Piece::canonical(bool show_color, bool show_letter) const
{
	std::string output;
	output += show_color ? (color ? "b":"w") : "";
	output += show_letter ? canon_name : "";
	return output;
}
