#include "pieces/pion.hpp"
#include "echiquier.hpp"
#include <iostream>

bool Pion::est_mouvement_legal(Square org, Square dest) const {

	int row_org, col_org, row_dest, col_dest;
	dest.get_row_col(row_dest, col_dest);
	org.get_row_col(row_org, col_org);

	// Cas de base
	if(!Piece::est_mouvement_legal(org, dest)) return false;

	int dir = get_color() ? -1 : 1;

	Piece* piece = echiquier->get_piece(dest);

	if(row_org+dir == row_dest && col_org == col_dest){
		// Cas du mouvement classique
		return piece == nullptr;
	}
	if((row_org == (row_dest - dir)) && ((col_org == (col_dest + dir)) || (col_org == (col_dest - dir)))){
		// Prise en passant
		if ((row_org == 4 && get_color() == WHITE) || (row_org == 3 && get_color() == BLACK)) {
			// On récupère le pion qui devrait être pris par la prise en passant
			Piece* target_piece = echiquier->get_piece(Square(row_dest - dir, col_dest));
			if (can_en_passant(target_piece)) return true;
		}

		// Cas où une pièce peut être prise
		return piece != nullptr;
	}
	if(row_org+2*dir == row_dest && col_org == col_dest){
		if(!get_has_moved())
			return piece == nullptr;
		return false;
	}
	return false;
}

void Pion::move_event(Square from){

	Piece::move_event(from);

	int row_org, dest_org, row_dest, col_dest;
	from.get_row_col(row_org, dest_org);
	get_pos().get_row_col(row_dest, col_dest);

	int dir = get_color() ? -1 : 1;

	if((get_color() == WHITE && row_dest == 7) || (get_color() == BLACK && row_dest == 0)){
		handle_promote();
	}else if ((row_org == 4 && get_color() == WHITE) || (row_org == 3 && get_color() == BLACK)) {
		// On récupère le pion qui devrait être pris par la prise en passant
		Piece* target_piece = echiquier->get_piece(Square(row_dest - dir, col_dest));
		if (can_en_passant(target_piece)) {
			echiquier->set_pos(nullptr, target_piece->get_pos());
			target_piece->set_is_active(false);
		}
	}
}

void Pion::handle_promote() {
	std::cout << std::endl << "En quelle pièce souhaitez vous promouvoir votre pion (Q, N, R, B) ?  ";
	bool flag = true;
	std::string choice;
	while (flag)
	{
		std::cin >> choice;
		if(choice == "Q" || choice == "N" || choice == "R" || choice == "B") echiquier->promote(this, choice), flag = false;
		else std::cout << "\x1b[2K" << "Choix invalide. Valeurs possibles : Q, N, R, B";
	}
}

bool Pion::can_en_passant(Piece* target_piece) const {
	// On récupère le pion qui devrait être pris par la prise en passant
	Move last_move = echiquier->get_moves().back();
	if (target_piece != nullptr && *target_piece == *last_move.get_moved_piece()) {
		// Si la pièce s'est déplacée de 2 cases
		if (abs(last_move.get_old_pos().get_row() - last_move.get_new_pos().get_row()) == 2) {
			// Vérification de si la pièce est un pion
			if (last_move.get_moved_piece()->get_id() >= 8 && last_move.get_moved_piece()->get_id() <= 15) {
				return true;
			}
		}
	}

	return false;
}

bool Pion::est_menace(Square org, Square dest) const {

	int row_org, col_org, row_dest, col_dest;
	dest.get_row_col(row_dest, col_dest);
	org.get_row_col(row_org, col_org);

	// Cas de base
	if(!Piece::est_mouvement_legal(org, dest)) return false;
	int dir = get_color() ? -1 : 1;
	Piece* piece = echiquier->get_piece(dest);
	if((row_org == (row_dest - dir)) && ((col_org == (col_dest + dir)) || (col_org == (col_dest - dir)))) 
		return piece != nullptr;
	else 
		return false;
}